package simplyconnect.jira

import com.atlassian.jira.rest.client.GetCreateIssueMetadataOptions
import com.atlassian.jira.rest.client.GetCreateIssueMetadataOptionsBuilder
import com.atlassian.jira.rest.client.IssueRestClient
import com.atlassian.jira.rest.client.JiraRestClient
import com.atlassian.jira.rest.client.NullProgressMonitor
import com.atlassian.jira.rest.client.domain.BasicIssue
import com.atlassian.jira.rest.client.domain.CimIssueType
import com.atlassian.jira.rest.client.domain.CimProject
import com.atlassian.jira.rest.client.domain.Comment
import com.atlassian.jira.rest.client.domain.EntityHelper
import com.atlassian.jira.rest.client.domain.Issue
import com.atlassian.jira.rest.client.domain.input.IssueInput
import com.atlassian.jira.rest.client.internal.jersey.JerseyJiraRestClientFactory
import groovy.json.JsonSlurper
import groovy.util.logging.Slf4j

/**
 * TODO start up a test instance of Jira and write some tests against it
 * User: david
 * Date: 03/02/13
 */
@Slf4j
class Jiranator {
    Credentials config
    IssueRestClient issueClient
    private static NullProgressMonitor progressMonitor = new NullProgressMonitor()

    Jiranator(Credentials config) {
        this.config = config
    }

    public String createIssue(IssueInput input) {
        // TODO add subject to the the following log statement
        log.info("Attempting to create issue")
        BasicIssue issue = issueClient.createIssue(input, progressMonitor)
        // TODO add subject to the the following log statement
        log.info("Created jira ${issue.key}")
        return issue.key
    }

    public String addComment(String comment, String jiraNumber) {
        Issue issue = find(jiraNumber)
        URI commentsUri = issue.getCommentsUri()
        log.info("adding comment to $jiraNumber with summary ${issue.summary}")
        issueClient.addComment(progressMonitor, commentsUri, Comment.valueOf(comment))
        log.info("added comment to $jiraNumber with summary ${issue.summary}")
    }

    public Issue find(String jiraNumber) {
        log.info("Searching for jira $jiraNumber")
        return getIssueClient().getIssue(jiraNumber, progressMonitor)
    }

    private IssueRestClient getIssueClient() {
        if (!issueClient) {
            log.info("Creating issue client")
            JerseyJiraRestClientFactory factory = new JerseyJiraRestClientFactory();
            JiraRestClient restClient = factory.createWithBasicHttpAuthentication(config.baseUrl, config.user, config.password);
            issueClient = restClient.getIssueClient()
        }
        return issueClient
    }

    CimProject findCimProject(String projectKey) {
        log.info("Searching for project $projectKey")
        // TODO - should we cache these?
        GetCreateIssueMetadataOptions issueMetadata = new GetCreateIssueMetadataOptionsBuilder().withProjectKeys(projectKey).withExpandedIssueTypesFields().build()
        Iterable<CimProject> projects = getIssueClient().getCreateIssueMetadata(issueMetadata, progressMonitor)
        return projects.iterator().next();
    }

    CimIssueType findCimIssueType(String jiraType, CimProject project) {
        log.info("Searching for issue type ${jiraType}")
        return EntityHelper.findEntityByName(project.getIssueTypes(), jiraType);
    }

    String findUserIdByEmail(String address) {
        log.info("Finding user id for email: $address")
        String urlString = JiraRestUrl.urlUserSearchByEmail(address)
        String userId = null
        try {
            String result = retrieve(urlString)
            if(result) {
                def json = new JsonSlurper().parseText(result)
                if(json.size() > 0){
                    userId = json[0].name
                }
            }
        } catch (Exception exc) {
            log.error("Could not locate user id for email address $address via url $urlString", exc)
        }
        log.info("User id for $address was $userId")
        return userId
    }

    private String retrieve(String urlString) {
        log.info("retrieving data via $urlString")
        URL url = new URL(urlString)
        def authString = "${config.user}:${config.password}".getBytes().encodeBase64().toString()
        URLConnection connection = url.openConnection()
        connection.setRequestProperty("Authorization", "Basic ${authString}")
        connection.connect()
        return connection.content.text
    }
}
